@@ -2,7 +2,7 @@
 /**
  *
  * Tennix! SDL Port
- * Copyright (C) 2003, 2007 Thomas Perl <thp@perli.net>
+ * Copyright (C) 2003, 2007, 2008 Thomas Perl <thp@perli.net>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,8 +34,8 @@ void game( bool singleplayer) {
     GameState s = {
         { 0, 0, 0.0, 0.0, 0.0 },
         { 0, 0, 0, 0, 0 },
-        { GAME_X_MIN-RACKET_X_MID*2, GAME_Y_MID, 0, 0, 1, DESIRE_NORMAL, PLAYER_TYPE_HUMAN, false },
-        { GAME_X_MAX+RACKET_X_MID*2, GAME_Y_MID, 0, 0, 0, DESIRE_NORMAL, PLAYER_TYPE_HUMAN, false },
+        { GAME_X_MIN-RACKET_X_MID*2, GAME_Y_MID, 0, 0, 1, DESIRE_NORMAL, PLAYER_TYPE_HUMAN, GAME_Y_MID, false, 0, {0}, 0, 0, PLAYER_ACCEL_DEFAULT, true },
+        { GAME_X_MAX+RACKET_X_MID*2, GAME_Y_MID, 0, 0, 0, DESIRE_NORMAL, PLAYER_TYPE_HUMAN, GAME_Y_MID, false, 0, {0}, 0, 0, PLAYER_ACCEL_DEFAULT, true },
         0,
         0,
         0,
@@ -48,6 +48,7 @@ void game( bool singleplayer) {
         WINNER_NONE,
         false,
         GR_CTT_GRASS,
+        -1,
         { 0 },
         0,
         false,
@@ -70,6 +71,10 @@ void game( bool singleplayer) {
     bool quit = false;
     int x, y, z;
 
+#ifdef ENABLE_FPS_LIMIT
+    Uint32 ft, frames; /* frame timer and frames */
+#endif
+
     if( singleplayer) {
 #ifdef DEBUG
         s.player1.type = PLAYER_TYPE_AI;
@@ -87,7 +92,11 @@ void game( bool singleplayer) {
             }
         }
     }
-   
+
+#ifdef ENABLE_FPS_LIMIT
+    frames = 0;
+    ft = SDL_GetTicks();
+#endif
     while( !quit) {
         nt = SDL_GetTicks();
         diff = nt-ot;
@@ -109,6 +118,13 @@ void game( bool singleplayer) {
             }
         }
 
+#ifdef ENABLE_FPS_LIMIT
+        while (frames*1000.0/((float)(SDL_GetTicks()-ft+1))>(float)(DEFAULT_FPS)) {
+            SDL_Delay(10);
+        }
+        frames++;
+#endif
+
         render( &s);
     }
    
@@ -224,16 +240,6 @@ bool step( GameState* s) {
         s->history_is_locked = false;
     }
 
-    /* Update ball_dest for debugging purposes */
-    get_move_y( s, 1);
-    get_move_y( s, 2);
-
-    s->ball.x += s->ball.move_x;
-    s->ball.y += s->ball.move_y;
-    
-    if( s->player1.state) s->player1.state--;
-    if( s->player2.state) s->player2.state--;
-
     SDL_PollEvent( &e);
     keys = SDL_GetKeyState( NULL);
     switch(e.type) {
@@ -264,13 +270,18 @@ bool step( GameState* s) {
                    keys['w'] || keys[SDLK_UP] || s->joystick_y < -JOYSTICK_TRESHOLD,
                    keys['s'] || keys[SDLK_DOWN] || s->joystick_y > JOYSTICK_TRESHOLD,
                    keys['d'] || keys[SDLK_SPACE] || keys[SDLK_LCTRL] || s->joystick_a,
+#ifdef ENABLE_MOUSE
+                   true,
+#else
+                   false,
+#endif
                    s);
         } else {
             input_ai( &s->player1, &s->ball, &s->player2, s);
         }
  
         if( s->player2.type == PLAYER_TYPE_HUMAN) {
-            input_human( &s->player2, keys['o'], keys['l'], keys['k'], s);
+            input_human( &s->player2, keys['o'], keys['l'], keys['k'], false, s);
         } else {
             input_ai( &s->player2, &s->ball, &s->player1, s);
         }
@@ -304,6 +315,16 @@ bool step( GameState* s) {
     limit_value( &s->player2.y, PLAYER_Y_MIN, PLAYER_Y_MAX);
     limit_value( &s->ball.jump, BALL_JUMP_MIN, BALL_JUMP_MAX);
     
+    /* Update ball_dest for debugging purposes */
+    get_move_y(s, 1);
+    get_move_y(s, 2);
+
+    s->ball.x += s->ball.move_x;
+    s->ball.y += s->ball.move_y;
+
+    if(s->player1.state) s->player1.state--;
+    if(s->player2.state) s->player2.state--;
+
     return false;
 }
 
@@ -317,16 +338,21 @@ void render( GameState* s) {
             start_fade();
             s->is_over = true;
         }
-        clearscr();
+        clear_screen();
+        store_screen();
         show_sprite( GR_RACKET, 2*(s->winner-1), 4, WIDTH/2 - get_image_width( GR_RACKET)/8, HEIGHT/2 - get_image_height( GR_RACKET), 255);
         sprintf( s->game_score_str, "player %d wins the match with %s", s->winner, format_sets( s));
         font_draw_string( GR_DKC2_FONT, s->game_score_str, (WIDTH-font_get_string_width( GR_DKC2_FONT, s->game_score_str))/2, HEIGHT/2 + 30, s->time/20, ANIMATION_WAVE | ANIMATION_BUNGEE);
         updatescr();
         return;
     }
-    clearscr();
-    fill_image( s->court_type, 120, 120, 400, 250);
-    show_image( GR_COURT, 0, 0, 255);
+    if (s->old_court_type != s->court_type) {
+        clear_screen();
+        fill_image(s->court_type, 120, 120, 400, 250);
+        show_image(GR_COURT, 0, 0, 255);
+        s->old_court_type = s->court_type;
+        store_screen();
+    }
     show_sprite( GR_REFEREE, s->referee, 4, 250, 10, 255);
     show_image( GR_SHADOW, s->ball.x-BALL_X_MID, s->ball.y + get_phase( s) - BALL_Y_MID, 255);
     
@@ -345,6 +371,11 @@ void render( GameState* s) {
         show_sprite( GR_BALL, 0, 4, s->ball.x-BALL_X_MID, s->ball.y-BALL_Y_MID, 255);
     }
 
+    /* Player 1's mouse rectangle */
+    if (!(s->player1.mouse_locked)) {
+        rectangle(s->player1.x-2+5, s->player1.mouse_y-2, 4, 4, 255, 255, 255);
+        rectangle(s->player1.x-1+5, s->player1.mouse_y-1, 2, 2, 0, 0, 0);
+    }
     
     font_draw_string( GR_DKC2_FONT, s->game_score_str, 14, 14, 0, ANIMATION_NONE);
     font_draw_string( GR_DKC2_FONT, s->sets_score_str, (WIDTH-font_get_string_width( GR_DKC2_FONT, s->sets_score_str))-14, 14, 0, ANIMATION_NONE);
@@ -434,8 +465,40 @@ float get_move_y( GameState* s, unsigned char player) {
     return (y_len*move_x)/(x_len);
 }
 
-void input_human( Player* player, bool up, bool down, bool hit, GameState* s) {
+void input_human( Player* player, bool up, bool down, bool hit, bool use_mouse, GameState* s) {
     int diff = PLAYER_MOVE_Y;
+    int mb;
+
+    /**
+     * Only use mouse control if the user isn't pressing any buttons
+     * this way, keyboard control still works when mouse control is
+     * enabled.
+     **/
+    if (use_mouse && (down || up || hit)) {
+        /**
+         * this is here so if the user decides to play
+         * with keyboard controls, we will lock the 
+         * mouse and disable displaying the mouse cursor
+         **/
+        player->mouse_locked = true;
+    }
+    if (use_mouse && !down && !up && !hit) {
+        mb = SDL_GetMouseState(&(player->mouse_x), &(player->mouse_y));
+        if (mb&SDL_BUTTON(SDL_BUTTON_LEFT)) {
+            if (player->mouse_y < player->y) {
+                down = false;
+                up = true;
+                diff = (player->y-player->mouse_y<diff)?(player->y-player->mouse_y):(diff);
+            } else if (player->mouse_y > player->y) {
+                up = false;
+                down = true;
+                diff = (player->mouse_y-player->y<diff)?(player->mouse_y-player->y):(diff);
+            }
+            player->mouse_locked = false;
+        } else if (!player->mouse_locked) {
+            hit = true;
+        }
+    }
 
     if (fabsf(s->joystick_y) > JOYSTICK_TRESHOLD) {
         diff = PLAYER_MOVE_Y*fabsf(s->joystick_y)/40.0;
@@ -444,12 +507,14 @@ void input_human( Player* player, bool up, bool down, bool hit, GameState* s) {
         }
     }
 
-    if( up) {
-        player->y -= diff;
-    }
-
-    if( down) {
-        player->y += diff;
+    if (up) {
+        player->y -= fminf(diff, diff*player->accelerate);
+        player->accelerate *= PLAYER_ACCEL_INCREASE;
+    } else if (down) {
+        player->y += fminf(diff, diff*player->accelerate);
+        player->accelerate *= PLAYER_ACCEL_INCREASE;
+    } else {
+        player->accelerate = PLAYER_ACCEL_DEFAULT;
     }
 
     if( hit) {
