@@ -2,7 +2,7 @@
 /**
  *
  * Tennix! SDL Port
- * Copyright (C) 2003, 2007 Thomas Perl <thp@perli.net>
+ * Copyright (C) 2003, 2007, 2008 Thomas Perl <thp@perli.net>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -71,6 +71,10 @@ typedef struct {
     bool state_locked; /* enabled when user keeps pressing the "hit" key */
     int game; /* score for the current game */
     int sets[SETS_TO_WIN*2]; /* score for each set */
+    int mouse_x; /* x position of mouse */
+    int mouse_y; /* y position of mouse */
+    float accelerate; /* a value [0..1] how fast the user accelerates */
+    bool mouse_locked; /* on start, ignore unpressed mouse state */
 } Player;
 
 enum {
@@ -100,6 +104,7 @@ typedef struct {
     int winner;
     bool is_over;
     unsigned int court_type;
+    unsigned int old_court_type;
     unsigned int history[3];
     unsigned int history_size;
     bool history_is_locked;
@@ -120,7 +125,6 @@ typedef struct {
 
 #define BALL_JUMP_MIN 4
 #define BALL_JUMP_MAX 10
-#define BALL_DEFAULT_SPEED 3.0
 
 #define RACKET_X_MID 15
 #define RACKET_Y_MID 24
@@ -156,19 +160,28 @@ typedef struct {
 #define IS_NEAR_X(px,bx) (fabsf(px-bx)<PLAYER_AREA_X)
 #define IS_NEAR_X_AI(px,bx) (fabsf(px-bx)<PLAYER_AREA_X*2)
 
-#define PLAYER_MOVE_Y 6.0
+#define PLAYER_MOVE_Y 5.0
+#define PLAYER_ACCEL_DEFAULT 0.2
+#define PLAYER_ACCEL_INCREASE 1.2
 #define PLAYER_STATE_MAX 7
 #define PLAYER_POWERSHOT 6.2
 
 #define MOVE_Y_SEED 3
 
+/* Comment out the following #define to enable mouse control */
+#define ENABLE_MOUSE
+
+/* Comment the following #define to disable FPS limiting */
+#define ENABLE_FPS_LIMIT
+#define DEFAULT_FPS 33
+
 void game( bool);
 void render( GameState*);
 bool step( GameState*);
 void limit_value( float*, float, float);
 float get_phase( GameState*);
 float get_move_y( GameState*, unsigned char);
-void input_human( Player*, bool, bool, bool, GameState*);
+void input_human( Player*, bool, bool, bool, bool, GameState*);
 void input_ai( Player*, Ball*, Player*, GameState*);
 void game_setup_serve( GameState*);
 float ngram_predictor( GameState*);
